var RELATIVELY_FAR_AWAY_YEAR = 2062;

enyo.kind({
	name: "ContactsList",
	kind: enyo.VFlexBox,	
	
	published: {
    	showsPhotos: true,
    	noImageUrl: "images/person.png"
  	},
	
	components:[
		
		//non-ui
		{kind:"pmeUtils", name:"utils"},
		{kind:"CalendarManager"},
		
		
		//ui
		{kind:"PageHeader", className: "enyo-header", components:[
			{kind:"Spacer"},
			{allowHtml:true, name:"todaysDate"},
			{kind:"Spacer"}
		]},
		
		
		//confirmation
		{kind: "ModalDialog", name:"confirmDialog", lazy:false, components: [
        	{name:"dialogContent", allowHtml:true, style: "padding-left: 10px"},
          	{layoutKind: "HFlexLayout", pack: "center", components: [
            	{kind: "Button", caption: "OK", onclick: "confirmClick"},
            	{kind: "Button", caption: "Cancel", onclick: "cancelClick"}
          	]}
      	]},
		
		
		
		//list
	    {kind: "VirtualList", onSetupRow: "setupRow", flex:1, components: [
	        {kind: "Item", layoutKind: "VFlexLayout", components: [
	        
	        	{name:"divider", content:"separator", className:"list-divider"},
	        	{kind:"Control", layoutKind:"HFlexLayout",components:[
	        		{name: "photo", kind:"Image", className:"square-list-photo", onerror:"noImageCallback"},
	        		{kind:"VFlexBox", flex:1, components:[
	            		{name: "name", allowHtml:true},
	            		{name: "birthday", allowHtml:true}
	            	]},
	            	{kind:"VFlexBox", components:[
	            		{kind:"Spacer"},
	            		{kind:"IconButton", onclick:"addReminderClick", icon: "images/clock.png"},
	            		{kind:"Spacer"}
	            	]}
	        	]}
	        ]}
	    ]}
    ],
    
    
    
    create:function(){
    	this.inherited(arguments);
    	this.noPhotoRows = [];
    	this.alarmData = [];
    	
    	var dateFmt = new enyo.g11n.DateFmt({date:"long"});
    	this.$.todaysDate.setContent("<b>"+dateFmt.format(new Date())+"</b>");
    	
    },
    
    
    
    setupRow: function(inSender, inIndex) {
    	
    	var recs = enyo.application.ContactsList;
    	
    	if (recs == null)
    		return;
    	
    	
    	
    	
	    if (inIndex < recs.length && inIndex >= 0) {
	    	
	    	//setting name
	        this.$.name.setContent('<b>'+
	        	(recs[inIndex].givenName.length>0 ? (recs[inIndex].givenName+ " ") : "") + 
	        	recs[inIndex].familyName+'</b>');
	 


	        //calculating and formatting date & age
	       	var bDayString = recs[inIndex].birthday;
	        bDayString = bDayString.replace("0000",RELATIVELY_FAR_AWAY_YEAR);
	        var date = new Date(bDayString);
	        var now = new Date();
	        var nextOccurenceYear = recs[inIndex].sortBirthday.substr(0,4);
	        var age = nextOccurenceYear - date.getFullYear();
	        
	        var dateFmt = new enyo.g11n.DateFmt({format:"MMMM d"});
	        
	        //setting birthday and age
	        this.$.birthday.setContent(dateFmt.format(date) + (age > 0 ? " (age "+age+")" : ""));
	        
	        
	        //if tomorrow - signal
	        date.setFullYear(now.getFullYear());
	       
	        var isTomorrow = (this.$.utils.daysBetween(date,now) > 0 && this.$.utils.daysBetween(date,now) < 1);
	        var isToday = (this.$.utils.daysBetween(date,now) < 0 && this.$.utils.daysBetween(date,now) > -1);
	        
	        if (isTomorrow)      
	        	this.$.birthday.setContent(this.$.birthday.content + " <font color='blue'>tomorrow</font>");
	        else if (isToday)      
	        	this.$.birthday.setContent(this.$.birthday.content + " <font color='red'>today</font>");
	        
	        
	        
	        //enable-disable divider if needed
	        if (recs[inIndex].isGroupHeader){
	        	this.prevMonthIndex = date.getMonth();
	        	dateFmt = new enyo.g11n.DateFmt({format:"MMMM"});
	        	this.$.divider.setContent(dateFmt.format(date) + " " + nextOccurenceYear);
	        	this.$.divider.show();
	        }
	        else{
	        	this.$.divider.hide();
	        }
	        
	        
	        //photo loading
	        if (this.showsPhotos){
	        	this.$.photo.show();
	        	if (this.noPhotoRows.indexOf(inIndex) > -1)
	        	{
	        		this.$.photo.setSrc(this.noImageUrl);
	        	}
	        	else
	        	{
	        		this.$.photo.setSrc(recs[inIndex].squarePhoto.length>0 ? recs[inIndex].squarePhoto : this.noImageUrl);
	        	}	
	        }
	        else{
	        	this.$.photo.hide();
	        }
	        return true;
	    }
	},
    
    
    reloadList: function(data){
    	this.$.virtualList.punt();
    },
    
    
    setShowsPhotos:function(inShowsPhotos){
    	this.showsPhotos = inShowsPhotos;
    	this.reloadList();
    },
    
    
    noImageCallback : function(inSender,inEvent){
    	//console.log('problem: '+inEvent.rowIndex);
    	// inSender.setSrc(this.noImageUrl);
    	this.noPhotoRows.push(inEvent.rowIndex);
    	this.$.virtualList.refresh();
    },
    
    
    addReminderClick: function(inSender, inEvent){
    	var recs = enyo.application.ContactsList;
    	var index = inEvent.rowIndex;
    	var person = recs[index];
    	
    	var name = (person.givenName.length>0 ? (person.givenName+ " ") : "") + person.familyName
    	
    	var date = new Date(person.sortBirthday);
    	
    	this.alarmData = {name:name, date:date};
    	
    	this.$.dialogContent.setContent('Add reminder for <b>'+name+"'s</b> birthday to Calendar?")
    	this.$.confirmDialog.openAtCenter();
    	
    },
    
    confirmClick: function(inSender, inEvent){
    	this.$.calendarManager.addEventForPerson(this.alarmData.name, this.alarmData.date);
    	this.cancelClick();
    },
    
    
    cancelClick: function(inSender, inEvent){
    	this.alarmData = {};
    	this.$.confirmDialog.close();
    }
});